// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.event_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart';

/// This variable is used to check that events are indeed processed
/// asynchronously.
var asyncSwitch = true;

main() {
  test('Send mEmail message', () {
    var bus = new Omnibus();
    bus.bindEventHandler(mEmailId, {'to': 'me@example.com'}, emailHandler);

    // Process asynchronous.
    bus.publishEvent(
        mEmail('Hi!', from: 'bad@example.com', to: 'notme@example.com'));
    bus.publishEvent(
        mEmail('Hi!', from: 'good@example.com', to: 'me@example.com'));
    asyncSwitch = false;

    // Await event using AsyncProcesses<Null>.
    bus
        .publishEvent(
            mEmail('Hi!', from: 'good@example.com', to: 'me@example.com'))
        .then((AsyncProcesses<Null> processes) async {
      await processes.onComplete().then((List<Null> processes) {
        asyncSwitch = true;

        // There should be one event handler that processed this event
        // (emailHandler)
        expect(processes.length, equals(1));
      });

      // Check if onComplete was executed.
      expect(asyncSwitch, equals(true));
    });
  });
}

Future emailHandler(OmnibusEvent event) async {
  // print("You've Got Mail!");
  // print("From:\t${email['from']}");
  // print("To:\t${email['to']}");
  // print("Body:\t${email['body']}");
  expect(asyncSwitch, equals(false));
  expect(event.data['to'], equals('me@example.com'));
}
