// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.exceptions_test;

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart';

/// JSON incompatible data
class JsonIncompatible {
  int number;
  JsonIncompatible(this.number) {}
}

/// toJson with JSON incompatible data
class UglyJsonIncompatible {
  int number;
  UglyJsonIncompatible(this.number) {}
  JsonIncompatible toJson() => new JsonIncompatible(number);
}

main() {
  test('Bind null event handler', () {
    expect(() {
      new Omnibus().bindEventHandler(mDataId, {}, null);
    }, throwsArgumentError);
  });

  test('Bind null request handler', () {
    expect(() {
      new Omnibus().bindRequestHandler(mFakeRequestId, {}, null);
    }, throwsArgumentError);
  });

  test('Bind null global event handler', () {
    expect(() {
      new Omnibus().bindGlobalEventHandler(null);
    }, throwsArgumentError);
  });

  test('Bind null global request handler', () {
    expect(() {
      new Omnibus().bindGlobalRequestHandler(null);
    }, throwsArgumentError);
  });

  test('Publish illegal data with JsonIncompatible', () {
    expect(() {
      new Omnibus().publishEvent(mData({'data': new JsonIncompatible(1)}));
    }, throwsNoSuchMethodError);
  });

  test('Publish illegal data with UglyJsonIncompatible', () {
    expect(() {
      new Omnibus().publishEvent(mData({'data': new UglyJsonIncompatible(1)}));
    }, throwsArgumentError);
  });
}
