// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.global_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart';

/// Counters for the global handlers
int totalEvents = 0;
int totalRequests = 0;

main() {
  test('Test global handlers', () async {
    var bus = new Omnibus();

    // Bind global handlers.
    bus.bindGlobalEventHandler(globalEventHandler);
    bus.bindGlobalRequestHandler(globalRequestHandler);

    // Run some events.
    bus.publishEvent(mMessageForAll('This message is for everyone!'));
    bus.publishEvent(mEmail('Hi!', from: 'a@here.com', to: 'b@here.com'));

    // Run some requests.
    bus.publishRequest(
        mCalculationRequest(1.0, 1.0, CalculationOperator.subtract));
    bus.publishRequest(
        mCalculationRequest(1.0, 1.0, CalculationOperator.multiply));

    // This request goes trough all json_map functions.
    bus.publishRequest(mFakeRequest({
      'message': 'This is fake!',
      'map': {
        '1to3': [1, 2, '3']
      }
    }));

    // Construct request by hand.
    bus.publishRequest(new OmnibusRequest('omnibus.mCustom', {}));

    // Wait for async completion.
    new Future.delayed(new Duration(seconds: 1), () {
      expect(totalEvents, equals(2));
      expect(totalRequests, equals(3));
    });
  });
}

/// Event handler that receives all events.
Future globalEventHandler(OmnibusEvent event) async {
  totalEvents++;
}

/// Request handler that receives all requests.
Future<List<Future<OmnibusResponse>>> globalRequestHandler(
    OmnibusRequest request) async {
  totalRequests++;
  return new List<Future<OmnibusResponse>>()
    ..add(new Future.value(mFakeResponse()));
}
