// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.json_data_test;

import 'package:test/test.dart';
import 'package:yaml/yaml.dart';
import 'package:omnibus/json_data.dart';

main() {
  test('Test _getJsonValue with YamlList', () {
    // Construct YamlList and demonstrate it throws an exception when you try
    // to overwrite an index.
    List<int> list = [0, 1, 2, 3, 4, 5];
    var yamlList = new YamlList.wrap(list);
    expect(() {
      yamlList[0] = 0;
    }, throwsUnsupportedError);

    // Convert to a JSON compatible List and demonstrate that this value is now
    // mutable (i.e. it has been converted to a normal type).
    var jsonList = getJsonValue(yamlList);
    expect(jsonList, isList);
    expect(jsonList, equals(list));
    jsonList[0] = 0;
  });

  test('Test _getJsonValue with YamlMap', () {
    // Construct YamlMap and demonstrate it throws an exception when you try
    // to overwrite a key.
    Map<String, String> map = {
      'name': 'John Doe',
      'email': 'john@doe.com',
      'website': 'http://johndoe.com'
    };
    var yamlMap = new YamlMap.wrap(map);
    expect(() {
      yamlMap['name'] = 'John Doe';
    }, throwsUnsupportedError);

    // Convert to a JSON compatible Map and demonstrate that this value is now
    // mutable (i.e. it has been converted to a normal type).
    var jsonMap = getJsonValue(yamlMap);
    expect(jsonMap, isMap);
    expect(jsonMap, equals(map));
    jsonMap['name'] = 'John Doe';
  });
}
