// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.request_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart';

main() {
  test('Run calculations using mCalculationRequest messages', () async {
    var bus = new Omnibus();

    // Bind calculation handlers.
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.add.index}, calcAddHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.subtract.index}, calcSubtractHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.multiply.index}, calcMultiplyHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.divide.index}, calcDivideHandler);

    // Request addition, synced retrieval style with the pure Set.
    var responses = await (await bus.publishRequest(
            mCalculationRequest(1.0, 1.0, CalculationOperator.add)))
        .onComplete();
    expect(responses.length, equals(1));
    responses
        .where((OmnibusResponse response) =>
            response.omnibusId == mCalculationResponseId)
        .forEach((OmnibusResponse response) {
      expect(response.data['result'], equals(2.0));
    });

    // Request addition, synced retrieval style with the AsyncProcesses class.
    bus
        .publishRequest(mCalculationRequest(1.0, 1.0, CalculationOperator.add))
        .then((AsyncResponses responses) {
      expect(responses.count, equals(1));
      responses.forEach(mCalculationResponseId, (Map<String, dynamic> data) {
        expect(data['result'], equals(2.0));
      });
    });

    // Request division.
    // This time do not use onComplete but run forEach immediately. In a
    // real-life scenario this can increase the overall performance since the
    // responses that are already collected can be processed while the remaining
    // responses are collected. This part also uses the Future API rather than
    // the await alias.
    bus
        .publishRequest(
            mCalculationRequest(12.0, 8.0, CalculationOperator.divide))
        .then((AsyncResponses responses) {
      responses.forEach(mCalculationResponseId,
          (Map<String, dynamic> response) {
        expect(response['result'], equals(1.5));
      });
    });
  });
}

Future<OmnibusResponse> calcAddHandler(OmnibusRequest request) async {
  return mCalculationResponse(
      new CalculationResult(request.data['a'] + request.data['b']));
}

Future<OmnibusResponse> calcSubtractHandler(OmnibusRequest request) async {
  return mCalculationResponse(
      new CalculationResult(request.data['a'] - request.data['b']));
}

Future<OmnibusResponse> calcMultiplyHandler(OmnibusRequest request) async {
  return mCalculationResponse(
      new CalculationResult(request.data['a'] * request.data['b']));
}

Future<OmnibusResponse> calcDivideHandler(OmnibusRequest request) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future.delayed(
      new Duration(seconds: 1),
      () => mCalculationResponse(
          new CalculationResult(request.data['a'] / request.data['b'])));
}
